# Synfig plugin: Animate text
# Helps animate text
# (c) 2023 Pablo Gil Fernández
# MIT License

from xml.etree import ElementTree
import sys
import json
import re

def create_fill_node(color_str):
    [r,g,b,a] = [0.,0.,0.,1.]
    if color_str.startswith('rgba'):
        color_str = color_str[5:-1]
        [r,g,b,a] = color_str.split(',')
    else:
        color_str = color_str[4:-1]
        [r,g,b] = color_str.split(',')

    node_str = '''<param name="color">
                    <color>
                        <r>{0}</r>
                        <g>{1}</g>
                        <b>{2}</b>
                        <a>{3}</a>
                    </color>
                </param>'''
    
    return node_str.format(int(r)/255.0, int(g)/255.0, int(b)/255.0, a)

def closest_multiple(target, multiple):
    lo_multiple = float(int(target/multiple))*multiple
    hi_multiple = lo_multiple + multiple
    distance_to_lo_multiple = abs(target - lo_multiple)
    distance_to_hi_multiple = abs(target - hi_multiple)
    
    if distance_to_lo_multiple < distance_to_hi_multiple:
        return lo_multiple
    else:
        return hi_multiple


synfig_filename = sys.argv[1]
plugindata_filename = sys.argv[2]
output_filename = sys.argv[1]

# Get canvas information
with open(synfig_filename, 'r') as file:
    for line in file:
        if line.startswith('<canvas'):
            match = re.search(r'view-box="([^"]+)"', line)
            if match:
                view_box_str = match.group(1)
            else:
                view_box_str = '-1.0 1.0 1.0 -1.0'
            match = re.search(r'fps="([^"]+)"', line)
            if match:
                fps_str = match.group(1)
            break  # Break the loop after finding the first line
view_box = view_box_str.split()
fps = float(fps_str)
single_frame_time  = 1/fps
canvas_x = float(view_box[2])*2
canvas_y = float(view_box[1])*2
# TODO add text size to popup
text_size = 0.5
# TODO make time starting point the time position


# Open JSON file
f = open(plugindata_filename)
 
# Load JSON object as a dictionary
data = json.load(f)

# Close file
f.close()

# Fetch pseudo-XPath of selected layers if any
layers = data['canvas_state']['selected_layers'];
layer_xpath = None
if layers:
    layer_xpath = layers[0]
    if '#' in layer_xpath:
        sys.stderr.write('I just want to complain...')
        sys.exit(1)

# Fetch data from dialog
data = data['dialog']

# Load XML of current synfig animation file
tree = ElementTree.parse(synfig_filename)
root = tree.getroot()

# Create a XML node by string (embed in a group layer)
node_str = ''

group_start ='''
<layer type="group" active="true" exclude_from_rendering="false" version="0.3" desc="group_name">
    <param name="z_depth">
        <real value="0.0000000000"/>
    </param>
    <param name="amount">
        <real value="1.0000000000"/>
    </param>
    <param name="blend_method">
        <integer value="0" static="true"/>
    </param>
    <param name="origin">
        <vector>
            <x>0.0000000000</x>
            <y>0.0000000000</y>
        </vector>
    </param>'''

group_transformation_start = '''
    <param name="transformation">
        <composite type="transformation">'''

group_transformation_offset_still = '''
            <offset>
                <vector>
                    <x>offset_x_end</x>
                    <y>offset_y_end</y>
                </vector>
            </offset>'''

group_transformation_offset_animated = '''
            <offset>
                <animated type="vector">
                    <waypoint time="animation_time_starts" before="clamped" after="clamped">
                        <vector guid="">
                            <x>offset_x_start</x>
                            <y>offset_y_start</y>
                        </vector>
                    </waypoint>
                    <waypoint time="animation_time_ends" before="clamped" after="clamped">
                        <vector guid="">
                            <x>offset_x_end</x>
                            <y>offset_y_end</y>
                        </vector>
                    </waypoint>
                </animated>
            </offset>'''

group_transformation_angle_still = '''
            <angle>
                <angle value="0.000000"/>
            </angle>'''

group_transformation_angle_animated = '''
            <angle>
                <animated type="angle">
                    <waypoint time="animation_time_starts" before="clamped" after="clamped">
                        <angle guid="" value="angle_start"/>
                    </waypoint>
                    <waypoint time="animation_time_ends" before="clamped" after="clamped">
                        <angle guid="" value="angle_end"/>
                    </waypoint>
                </animated>
            </angle>'''

group_transformation_skew_still = '''
            <skew_angle>
                <angle value="0.000000"/>
            </skew_angle>'''

group_transformation_skew_animated = '''
            <skew_angle>
                <animated type="angle">
                    <waypoint time="animation_time_starts" before="clamped" after="clamped">
                        <angle guid="" value="angle_start"/>
                    </waypoint>
                    <waypoint time="animation_time_ends" before="clamped" after="clamped">
                        <angle guid="" value="angle_end"/>
                    </waypoint>
                </animated>
            </skew_angle>'''

group_transformation_scale_still = '''
            <scale>
                <vector>
                    <x>1.0000000000</x>
                    <y>1.0000000000</y>
                </vector>
            </scale>'''

group_transformation_scale_animated = '''
            <scale>
                <animated type="vector">
                <waypoint time="animation_time_starts" before="clamped" after="clamped">
                    <vector guid="">
                    <x>scale_x_start</x>
                    <y>scale_y_start</y>
                    </vector>
                </waypoint>
                <waypoint time="animation_time_ends" before="clamped" after="clamped">
                    <vector guid="">
                    <x>scale_x_end</x>
                    <y>scale_y_end</y>
                    </vector>
                </waypoint>
                </animated>
            </scale>'''

group_transformation_end = '''
        </composite>
    </param>'''

group_canvas_start = '''
    <param name="canvas">
        <canvas>'''

text_layer_start = '''
            <layer type="text" active="true" exclude_from_rendering="false" version="0.5" desc="my_text">
                <param name="z_depth">
                    <real value="0.0000000000"/>
                </param>
                <param name="blend_method">
                    <integer value="0" static="true"/>
                </param>
                <param name="text">
                    <string>my_text</string>
                </param>
                <param name="family">
                    <string>Arial</string>
                </param>
                <param name="style">
                    <integer value="0" static="true"/>
                </param>
                <param name="weight">
                    <integer value="400" static="true"/>
                </param>
                <param name="direction">
                    <integer value="0" static="true"/>
                </param>
                <param name="compress">
                    <real value="1.0000000000"/>
                </param>
                <param name="vcompress">
                    <real value="1.0000000000"/>
                </param>
                <param name="size">
                    <vector>
                    <x>0.5000000000</x>
                    <y>0.5000000000</y>
                    </vector>
                </param>
                <param name="orient">
                    <vector>
                    <x>0.5000000000</x>
                    <y>1.0000000000</y>
                    </vector>
                </param>
                <param name="use_kerning">
                    <bool value="true"/>
                </param>
                <param name="grid_fit">
                    <bool value="false"/>
                </param>
                <param name="invert">
                    <bool value="false"/>
                </param>'''

text_layer_opacity_still = '''
                <param name="amount">
                    <real value="1.0000000000"/>
                </param>'''

text_layer_opacity_animated = '''
                <param name="amount">
                    <animated type="real">
                        <waypoint time="animation_time_starts" before="clamped" after="clamped">
                        <real guid="" value="opacity_start"/>
                        </waypoint>
                        <waypoint time="animation_time_ends" before="clamped" after="clamped">
                        <real guid="" value="opacity_end"/>
                        </waypoint>
                    </animated>
                </param>'''

text_layer_origin_still = '''
                <param name="origin">
                    <vector>
                        <x>position_x_end</x>
                        <y>position_y_end</y>
                    </vector>
                </param>'''

text_layer_origin_animated = '''
                <param name="origin">
                    <animated type="vector">
                        <waypoint time="animation_time_starts" before="clamped" after="clamped">
                            <vector guid="">
                                <x>position_x_start</x>
                                <y>position_y_start</y>
                            </vector>
                        </waypoint>
                        <waypoint time="animation_time_ends" before="clamped" after="clamped">
                            <vector guid="">
                                <x>position_x_end</x>
                                <y>position_y_end</y>
                            </vector>
                        </waypoint>
                    </animated>
                </param>'''

text_layer_end = '''
            </layer>'''

group_canvas_end = '''
        </canvas>
    </param>'''

group_end = '''
    <param name="time_dilation">
        <real value="1.0000000000"/>
    </param>
    <param name="time_offset">
        <time value="0s"/>
    </param>
    <param name="children_lock">
        <bool value="false" static="true"/>
    </param>
    <param name="outline_grow">
        <real value="0.0000000000"/>
    </param>
    <param name="z_range">
        <bool value="false" static="true"/>
    </param>
    <param name="z_range_position">
        <real value="0.0000000000"/>
    </param>
    <param name="z_range_depth">
         <real value="0.0000000000"/>
    </param>
    <param name="z_range_blur">
         <real value="0.0000000000"/>
    </param>
</layer>'''

group_name = 'Group'
position_x_start = '0.0000000000'
position_y_start = '0.0000000000'
my_text = str(data['my_entry'])
animation_time = str(data['time'])
text_layer_color = create_fill_node(data['color'])

# start writing the node
def text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color):
    node_str = ''
    node_str += group_start
    node_str += group_transformation_start
    if trans_offset == 1:
        node_str += group_transformation_offset_animated
    else:
        node_str += group_transformation_offset_still
    if trans_angle == 1:
        node_str += group_transformation_angle_animated
    else:
        node_str += group_transformation_angle_still
    if trans_skew == 1:
        node_str += group_transformation_skew_animated
    else:
        node_str += group_transformation_skew_still
    if trans_scale == 1:
        node_str += group_transformation_scale_animated
    else:
        node_str += group_transformation_scale_still
    node_str += group_transformation_end
    node_str += group_canvas_start
    node_str += text_layer_start
    if text_opacity == 1:
        node_str += text_layer_opacity_animated
    else:
        node_str += text_layer_opacity_still
    if text_origin == 1:
        node_str += text_layer_origin_animated
    else:
        node_str += text_layer_origin_still
    if text_color == 1:
        # TODO
        node_str += text_layer_color
    else:
        node_str += text_layer_color

    node_str += text_layer_end
    node_str += group_canvas_end
    node_str += group_end
    return node_str

match data['animation_type']:
    case 'fade_in':
        if data['animated'] == '0':
            group_name = 'Text - fade in'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,1           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'opacity_start', '0.0000000000', node_str)
            node_str = re.sub(r'opacity_end', '1.0000000000', node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)
        else:
            group_name = 'Animated text - fade in'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str += text_layer_origin_still
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end

            node_str = re.sub(r'opacity_start', '0.0000000000', node_str)
            node_str = re.sub(r'opacity_end', '1.0000000000', node_str)

    case 'fade_out':
        if data['animated'] == '0':
            group_name = 'Text - fade out'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,1          ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'opacity_start', '1.0000000000', node_str)
            node_str = re.sub(r'opacity_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)
        else:
            group_name = 'Animated text - fade out'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str += text_layer_origin_still
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end

            node_str = re.sub(r'opacity_start', '1.0000000000', node_str)
            node_str = re.sub(r'opacity_end', '0.0000000000', node_str)

    case 'fly_up':
        if data['animated'] == '0':
            group_name = 'Text - fly up'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,0           ,1          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_start', str(canvas_y/2+0.75), node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)
        else:
            group_name = 'Animated text - fly up'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_start', str(canvas_y/2+0.75), node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end


    case 'fly_down':
        if data['animated'] == '0':
            group_name = 'Text - fly down'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,0           ,1          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_start', str(-canvas_y/2-0.75), node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)
        else:
            group_name = 'Animated text - fly down'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_start', str(-canvas_y/2-0.75), node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end

    case 'fly_left':
        if data['animated'] == '0':
            group_name = 'Text - fly left'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,0           ,1          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'position_x_start', str(-canvas_x), node_str)
            node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - fly left'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', str(-len(my_text)/4+position*text_size-canvas_x), node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            
    case 'fly_right':
        if data['animated'] == '0':
            group_name = 'Text - fly right'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,0          ,0           ,1          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'position_x_start', str(canvas_x), node_str)
            node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - fly right'
            node_str = group_start
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_animated
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', str(-len(my_text)/4+position*text_size+canvas_x), node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                position -= 1
            node_str += group_canvas_end
            node_str += group_end

    case 'pinwheel':
        if data['animated'] == '0':
            group_name = 'Text - pinwheel'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,1          ,0         ,1          ,0           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'angle_start', '-360.000000', node_str)
            node_str = re.sub(r'angle_end', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - pinwheel'
            node_str = group_start
            node_str = re.sub(r'group_name', group_name, node_str)
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str = re.sub(r'offset_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += group_start
                node_str = re.sub(r'group_name', 'Group - '+i, node_str)
                node_str += group_transformation_start
                node_str += group_transformation_offset_still
                node_str = re.sub(r'offset_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
                node_str += group_transformation_angle_animated
                node_str += group_transformation_skew_still
                node_str += group_transformation_scale_animated
                node_str += group_transformation_end
                node_str += group_canvas_start
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_still
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                node_str += group_canvas_end
                node_str += group_end
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            node_str = re.sub(r'angle_start', '-360.000000', node_str)
            node_str = re.sub(r'angle_end', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)

    case 'zoom_in':
        if data['animated'] == '0':
            group_name = 'Text - zoom in'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,1          ,0           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - zoom in'
            node_str = group_start
            node_str = re.sub(r'group_name', group_name, node_str)
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str = re.sub(r'offset_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += group_start
                node_str = re.sub(r'group_name', 'Group - '+i, node_str)
                node_str += group_transformation_start
                node_str += group_transformation_offset_still
                node_str = re.sub(r'offset_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
                node_str += group_transformation_angle_still
                node_str += group_transformation_skew_still
                node_str += group_transformation_scale_animated
                node_str += group_transformation_end
                node_str += group_canvas_start
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_still
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                node_str += group_canvas_end
                node_str += group_end
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)

    case 'zoom_out':
        if data['animated'] == '0':
            group_name = 'Text - zoom out'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,1          ,0           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'scale_x_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - zoom out'
            node_str = group_start
            node_str = re.sub(r'group_name', group_name, node_str)
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str = re.sub(r'offset_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += group_start
                node_str = re.sub(r'group_name', 'Group - '+i, node_str)
                node_str += group_transformation_start
                node_str += group_transformation_offset_still
                node_str = re.sub(r'offset_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
                node_str += group_transformation_angle_still
                node_str += group_transformation_skew_still
                node_str += group_transformation_scale_animated
                node_str += group_transformation_end
                node_str += group_canvas_start
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_still
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                node_str += group_canvas_end
                node_str += group_end
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            node_str = re.sub(r'scale_x_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '0.0000000000', node_str)

    case 'strech_horizontally':
        if data['animated'] == '0':
            group_name = 'Text - strech horizontally'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,1          ,0           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - strech horizontally'
            node_str = group_start
            node_str = re.sub(r'group_name', group_name, node_str)
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str = re.sub(r'offset_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += group_start
                node_str = re.sub(r'group_name', 'Group - '+i, node_str)
                node_str += group_transformation_start
                node_str += group_transformation_offset_still
                node_str = re.sub(r'offset_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
                node_str += group_transformation_angle_still
                node_str += group_transformation_skew_still
                node_str += group_transformation_scale_animated
                node_str += group_transformation_end
                node_str += group_canvas_start
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_still
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                node_str += group_canvas_end
                node_str += group_end
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            node_str = re.sub(r'scale_x_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)

    case 'strech_vertically':
        if data['animated'] == '0':
            group_name = 'Text - strech vertically'
            #          text_animation(trans_offset,trans_angle,trans_skew,trans_scale,text_opacity,text_origin,text_color)
            node_str = text_animation(0           ,0          ,0         ,1          ,0           ,0          ,0)

            node_str = re.sub(r'animation_time_start', '0.0000000000', node_str)
            node_str = re.sub(r'animation_time_end', animation_time, node_str)
            node_str = re.sub(r'scale_x_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)
            node_str = re.sub(r'my_text', my_text, node_str)

        else:
            group_name = 'Animated text - strech_vertically'
            node_str = group_start
            node_str = re.sub(r'group_name', group_name, node_str)
            node_str += group_transformation_start
            node_str += group_transformation_offset_still
            node_str = re.sub(r'offset_x_end', '0.0000000000', node_str)
            node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
            node_str += group_transformation_angle_still
            node_str += group_transformation_skew_still
            node_str += group_transformation_scale_still
            node_str += group_transformation_end
            node_str += group_canvas_start
            position = len(my_text)-1
            for i in reversed(my_text):
                node_str += group_start
                node_str = re.sub(r'group_name', 'Group - '+i, node_str)
                node_str += group_transformation_start
                node_str += group_transformation_offset_still
                node_str = re.sub(r'offset_x_end', str(-len(my_text)/4+position*text_size), node_str)
                node_str = re.sub(r'offset_y_end', '0.0000000000', node_str)
                node_str += group_transformation_angle_still
                node_str += group_transformation_skew_still
                node_str += group_transformation_scale_animated
                node_str += group_transformation_end
                node_str += group_canvas_start
                node_str += text_layer_start
                node_str += text_layer_opacity_still
                node_str += text_layer_origin_still
                if float(animation_time)/len(my_text) < single_frame_time:
                    animation_step = single_frame_time
                else:
                    animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
                node_str = re.sub(r'animation_time_start', str(position*animation_step), node_str)
                node_str = re.sub(r'animation_time_end', str((position+1)*animation_step), node_str)
                node_str += text_layer_color
                node_str += text_layer_end
                node_str = re.sub(r'position_x_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_start', '0.0000000000', node_str)
                node_str = re.sub(r'position_x_end', '0.0000000000', node_str)
                node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
                node_str = re.sub(r'my_text', i, node_str)
                node_str += group_canvas_end
                node_str += group_end
                position -= 1
            node_str += group_canvas_end
            node_str += group_end
            node_str = re.sub(r'scale_x_start', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_start', '0.0000000000', node_str)
            node_str = re.sub(r'scale_x_end', '1.0000000000', node_str)
            node_str = re.sub(r'scale_y_end', '1.0000000000', node_str)

    case 'typewriter':
        group_name = 'Text - typewritter'
        node_str = group_start
        node_str += group_transformation_start
        node_str += group_transformation_offset_still
        node_str += group_transformation_angle_still
        node_str += group_transformation_skew_still
        node_str += group_transformation_scale_still
        node_str += group_transformation_end
        node_str += group_canvas_start
        position = len(my_text)-1
        # Text animation
        for i in reversed(my_text):
            node_str += text_layer_start
            node_str += text_layer_opacity_animated
            node_str = re.sub(r'clamped', 'constant', node_str)
            if float(animation_time)/len(my_text) < single_frame_time:
                animation_step = single_frame_time
            else:
                animation_step  = closest_multiple(float(animation_time)/len(my_text),single_frame_time)
            node_str += text_layer_origin_still
            node_str += text_layer_color
            node_str += text_layer_end
            node_str = re.sub(r'animation_time_start', str((position+3)*animation_step), node_str)
            node_str = re.sub(r'animation_time_end', str((position+1+3)*animation_step), node_str)
            node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
            node_str = re.sub(r'position_y_end', '0.0000000000', node_str)
            node_str = re.sub(r'my_text', i, node_str)
            position -= 1
        node_str = re.sub(r'clamped', 'constant', node_str)
        node_str = re.sub(r'opacity_start', '0.0000000000', node_str)
        node_str = re.sub(r'opacity_end', '1.0000000000', node_str)

        # Prompt animation
        node_str += text_layer_start
        node_str = re.sub(r'my_text', '_', node_str)
        node_str += '''
                <param name="amount">
                    <animated type="real">'''
        for i in range(5):
            if i % 2 == 0:
                opacity = '1.0000000000'
            else:
                opacity = '0.0000000000'
            node_str += '''
                    <waypoint time="animation_times" before="constant" after="constant">
                    <real guid="" value="opacity"/>
                    </waypoint>'''
            node_str = re.sub(r'animation_time', str(animation_step*i), node_str)
            node_str = re.sub(r'opacity', opacity, node_str)
        node_str += '''
                    </animated>
                </param>
                <param name="origin">
                    <animated type="vector">
                        <waypoint time="0s" before="constant" after="constant">
                            <vector guid="">
                                <x>position_x_end</x>
                                <y>0.0000000000</y>
                            </vector>
                        </waypoint>'''
        node_str = re.sub(r'position_x_end', str(-len(my_text)/4), node_str)
        position = 1
        for i in range(len(my_text)):
            node_str += '''
                        <waypoint time="animation_times" before="constant" after="constant">
                            <vector guid="">
                                <x>position_x_end</x>
                                <y>0.0000000000</y>
                            </vector>
                        </waypoint>'''
            node_str = re.sub(r'animation_time', str(animation_step*(i+4)), node_str)
            node_str = re.sub(r'position_x_end', str(-len(my_text)/4+position*text_size), node_str)
            node_str = re.sub(r'opacity', opacity, node_str)  
            position += 1    
        node_str += '''
                    </animated>
                </param>'''
        node_str += text_layer_color
        node_str += text_layer_end
        node_str += group_canvas_end
        node_str += group_end



# Make some final text substitutions
node_str = re.sub(r'group_name', group_name, node_str)

node = ElementTree.fromstring(node_str)

# Get the index
parent_node = root # the canvas root node
index = -1

if not layer_xpath:
    if data['position'] == '1':
        index = 0
else:
    if layer_xpath[0] == ':':
        layer_xpath = '.' + layer_xpath[1:]
    layer_node = root.find(layer_xpath)
    parent_node = root.find(layer_xpath + '...')
    count = 0
    for nd in parent_node:
        if nd == layer_node:
            index = count
        count = count + 1
    if data['position'] == '0':
        index = index + 1
    
# Insert the new node
if index < 0:
    parent_node.append(node)
else:
    parent_node.insert(index, node)

# Write everything in file
tree.write(output_filename, 'UTF-8')

